@@ -1,4 +1,3 @@
-
 #
 # "Tax the rat farms." - Lord Vetinari
 #
@@ -16,41 +15,43 @@ package Math::BigRat;
 # anything older is untested, and unlikely to work
 use 5.006;
 use strict;
+use warnings;
+
 use Carp ();
 
 use Math::BigFloat;
-use vars qw($VERSION @ISA $upgrade $downgrade
-            $accuracy $precision $round_mode $div_scale $_trap_nan $_trap_inf);
+
+our ($VERSION, @ISA, $upgrade, $downgrade,
+     $accuracy, $precision, $round_mode, $div_scale, $_trap_nan, $_trap_inf);
 
 @ISA = qw(Math::BigFloat);
 
-$VERSION = '0.260801';
+$VERSION = '0.260802';
 $VERSION = eval $VERSION;
 
-# inherit overload from Math::BigFloat, but disable the bitwise ops that don't
-# make much sense for rationals unless they're truncated or something first
+# Inherit overload from Math::BigFloat, but disable the bitwise ops that don't
+# make much sense for rationals unless they're truncated or something first.
 
 use overload
-    map {
-	my $op = $_;
-	($op => sub {
-	    Carp::croak("bitwise operation $op not supported in Math::BigRat");
-	});
-    } qw(& | ^ ~ << >> &= |= ^= <<= >>=);
+  map {
+      my $op = $_;
+      ($op => sub {
+           Carp::croak("bitwise operation $op not supported in Math::BigRat");
+       });
+  } qw(& | ^ ~ << >> &= |= ^= <<= >>=);
 
-BEGIN
-  {
-  *objectify = \&Math::BigInt::objectify; 	# inherit this from BigInt
-  *AUTOLOAD = \&Math::BigFloat::AUTOLOAD;	# can't inherit AUTOLOAD
-  # we inherit these from BigFloat because currently it is not possible
-  # that MBF has a different $MBI variable than we, because MBF also uses
-  # Math::BigInt::config->('lib'); (there is always only one library loaded)
-  *_e_add = \&Math::BigFloat::_e_add;
-  *_e_sub = \&Math::BigFloat::_e_sub;
-  *as_int = \&as_number;
-  *is_pos = \&is_positive;
-  *is_neg = \&is_negative;
-  }
+BEGIN {
+    *objectify = \&Math::BigInt::objectify;  # inherit this from BigInt
+    *AUTOLOAD  = \&Math::BigFloat::AUTOLOAD; # can't inherit AUTOLOAD
+    # We inherit these from BigFloat because currently it is not possible
+    # that MBF has a different $MBI variable than we, because MBF also uses
+    # Math::BigInt::config->('lib'); (there is always only one library loaded)
+    *_e_add = \&Math::BigFloat::_e_add;
+    *_e_sub = \&Math::BigFloat::_e_sub;
+    *as_int = \&as_number;
+    *is_pos = \&is_positive;
+    *is_neg = \&is_negative;
+}
 
 ##############################################################################
 # Global constants and flags. Access these only via the accessor methods!
@@ -70,17 +71,22 @@ $_trap_inf = 0;                         # are infs ok? set w/ config()
 # Math::BigInt->config()->{lib}
 my $MBI = 'Math::BigInt::Calc';
 
-my $nan = 'NaN';
+my $nan   = 'NaN';
 my $class = 'Math::BigRat';
 
-sub isa
-  {
-  return 0 if $_[1] =~ /^Math::Big(Int|Float)/;		# we aren't
-  UNIVERSAL::isa(@_);
-  }
+sub isa {
+    return 0 if $_[1] =~ /^Math::Big(Int|Float)/; # we aren't
+    UNIVERSAL::isa(@_);
+}
 
 ##############################################################################
 
+# If $x is a Math::BigRat object and $f is a Math::BigFloat object, then
+#
+#   $x -> _new_from_float($f)
+#
+# converts $x into a Math::BigRat with the value of $f.
+
 sub _new_from_float
   {
   # turn a single float input into a rational number (like '0.1')
@@ -89,258 +95,262 @@ sub _new_from_float
   return $self->bnan() if $f->is_nan();
   return $self->binf($f->{sign}) if $f->{sign} =~ /^[+-]inf$/;
 
-  $self->{_n} = $MBI->_copy( $f->{_m} );	# mantissa
+  $self->{_n} = $MBI->_copy($f->{_m});	# mantissa
   $self->{_d} = $MBI->_one();
   $self->{sign} = $f->{sign} || '+';
   if ($f->{_es} eq '-')
     {
     # something like Math::BigRat->new('0.1');
     # 1 / 1 => 1/10
-    $MBI->_lsft ( $self->{_d}, $f->{_e} ,10);
+    $MBI->_lsft($self->{_d}, $f->{_e} ,10);
     }
   else
     {
     # something like Math::BigRat->new('10');
     # 1 / 1 => 10/1
-    $MBI->_lsft ( $self->{_n}, $f->{_e} ,10) unless
+    $MBI->_lsft($self->{_n}, $f->{_e} ,10) unless
       $MBI->_is_zero($f->{_e});
     }
-  $self;
+  return $self -> bnorm();
   }
 
-sub new
-  {
-  # create a Math::BigRat
-  my $class = shift;
+# If $x is a Math::BigRat object and $i is a Math::BigInt object, then
+#
+#   $x -> _new_from_int($i)
+#
+# converts $x into a Math::BigRat with the value of $i.
 
-  my ($n,$d) = @_;
+sub _new_from_int {
+    my ($self, $i) = @_;
 
-  my $self = { }; bless $self,$class;
+    return $self -> bnan()             if $i -> is_nan();
+    return $self -> binf($i -> sign()) if $i -> is_inf();
 
-  # input like (BigInt) or (BigFloat):
-  if ((!defined $d) && (ref $n) && (!$n->isa('Math::BigRat')))
-    {
-    if ($n->isa('Math::BigFloat'))
-      {
-      $self->_new_from_float($n);
-      }
-    if ($n->isa('Math::BigInt'))
-      {
-      # TODO: trap NaN, inf
-      $self->{_n} = $MBI->_copy($n->{value});		# "mantissa" = N
-      $self->{_d} = $MBI->_one();			# d => 1
-      $self->{sign} = $n->{sign};
-      }
-    if ($n->isa('Math::BigInt::Lite'))
-      {
-      # TODO: trap NaN, inf
-      $self->{sign} = '+'; $self->{sign} = '-' if $$n < 0;
-      $self->{_n} = $MBI->_new(abs($$n));		# "mantissa" = N
-      $self->{_d} = $MBI->_one();			# d => 1
-      }
-    return $self->bnorm();				# normalize (120/1 => 12/10)
-    }
-
-  # input like (BigInt,BigInt) or (BigLite,BigLite):
-  if (ref($d) && ref($n))
-    {
-    # do N first (for $self->{sign}):
-    if ($n->isa('Math::BigInt'))
-      {
-      # TODO: trap NaN, inf
-      $self->{_n} = $MBI->_copy($n->{value});		# "mantissa" = N
-      $self->{sign} = $n->{sign};
-      }
-    elsif ($n->isa('Math::BigInt::Lite'))
-      {
-      # TODO: trap NaN, inf
-      $self->{sign} = '+'; $self->{sign} = '-' if $$n < 0;
-      $self->{_n} = $MBI->_new(abs($$n));		# "mantissa" = $n
-      }
-    else
-      {
-      require Carp;
-      Carp::croak(ref($n) . " is not a recognized object format for Math::BigRat->new");
-      }
-    # now D:
-    if ($d->isa('Math::BigInt'))
-      {
-      # TODO: trap NaN, inf
-      $self->{_d} = $MBI->_copy($d->{value});		# "mantissa" = D
-      # +/+ or -/- => +, +/- or -/+ => -
-      $self->{sign} = $d->{sign} ne $self->{sign} ? '-' : '+';
-      }
-    elsif ($d->isa('Math::BigInt::Lite'))
-      {
-      # TODO: trap NaN, inf
-      $self->{_d} = $MBI->_new(abs($$d));		# "mantissa" = D
-      my $ds = '+'; $ds = '-' if $$d < 0;
-      # +/+ or -/- => +, +/- or -/+ => -
-      $self->{sign} = $ds ne $self->{sign} ? '-' : '+';
-      }
-    else
-      {
-      require Carp;
-      Carp::croak(ref($d) . " is not a recognized object format for Math::BigRat->new");
-      }
-    return $self->bnorm();				# normalize (120/1 => 12/10)
-    }
-  return $n->copy() if ref $n;				# already a BigRat
-
-  if (!defined $n)
-    {
-    $self->{_n} = $MBI->_zero();			# undef => 0
-    $self->{_d} = $MBI->_one();
-    $self->{sign} = '+';
+    $self -> {_n}   = $MBI -> _copy($i -> {value});
+    $self -> {_d}   = $MBI -> _one();
+    $self -> {sign} = $i -> {sign};
     return $self;
+}
+
+sub new {
+    my $self    = shift;
+    my $selfref = ref $self;
+    my $class   = $selfref || $self;
+
+    # Get numerator and denominator.
+
+    my ($n, $d) = @_;
+
+    # If called as a class method, initialize a new object.
+
+    $self = bless {}, $class unless $selfref;
+
+    # Input like $class->new($n), where there is no denominator, and where $n
+    # is a Math::BigInt or Math::BigFloat.
+
+    if ((!defined $d) && (ref $n) && (!$n->isa('Math::BigRat'))) {
+        if ($n->isa('Math::BigFloat')) {
+            $self->_new_from_float($n);
+        }
+        elsif ($n->isa('Math::BigInt')) {
+            # TODO: trap NaN, inf
+            $self->{_n} = $MBI->_copy($n->{value}); # "mantissa" = N
+            $self->{_d} = $MBI->_one();             # d => 1
+            $self->{sign} = $n->{sign};
+        }
+        elsif ($n->isa('Math::BigInt::Lite')) {
+            # TODO: trap NaN, inf
+            $self->{sign} = '+';
+            $self->{sign} = '-' if $$n < 0;
+            $self->{_n} = $MBI->_new(abs($$n)); # "mantissa" = N
+            $self->{_d} = $MBI->_one();         # d => 1
+        }
+        return $self->bnorm();  # normalize (120/100 => 6/5)
     }
 
-  # string input with / delimiter
-  if ($n =~ /\s*\/\s*/)
-    {
-    return $class->bnan() if $n =~ /\/.*\//;	# 1/2/3 isn't valid
-    return $class->bnan() if $n =~ /\/\s*$/;	# 1/ isn't valid
-    ($n,$d) = split (/\//,$n);
-    # try as BigFloats first
-    if (($n =~ /[\.eE]/) || ($d =~ /[\.eE]/))
-      {
-      local $Math::BigFloat::accuracy = undef;
-      local $Math::BigFloat::precision = undef;
+    # Input like $class->new($n, $d) where $n and $d both are Math::BigInt
+    # objects or Math::BigInt::Lite objects.
+    if (ref($d) && ref($n)) {
 
-      # one of them looks like a float
-      my $nf = Math::BigFloat->new($n,undef,undef);
-      $self->{sign} = '+';
-      return $self->bnan() if $nf->is_nan();
-
-      $self->{_n} = $MBI->_copy( $nf->{_m} );	# get mantissa
-
-      # now correct $self->{_n} due to $n
-      my $f = Math::BigFloat->new($d,undef,undef);
-      return $self->bnan() if $f->is_nan();
-      $self->{_d} = $MBI->_copy( $f->{_m} );
-
-      # calculate the difference between nE and dE
-      my $diff_e = $nf->exponent()->bsub( $f->exponent);
-      if ($diff_e->is_negative())
-	{
-        # < 0: mul d with it
-        $MBI->_lsft( $self->{_d}, $MBI->_new( $diff_e->babs()), 10);
-	}
-      elsif (!$diff_e->is_zero())
-        {
-        # > 0: mul n with it
-        $MBI->_lsft( $self->{_n}, $MBI->_new( $diff_e), 10);
+        # do N first (for $self->{sign}):
+        if ($n->isa('Math::BigInt')) {
+            # TODO: trap NaN, inf
+            $self->{_n} = $MBI->_copy($n->{value}); # "mantissa" = N
+            $self->{sign} = $n->{sign};
         }
-      }
-    else
-      {
-      # both d and n look like (big)ints
-
-      $self->{sign} = '+';					# no sign => '+'
-      $self->{_n} = undef;
-      $self->{_d} = undef;
-      if ($n =~ /^([+-]?)0*([0-9]+)\z/)				# first part ok?
-	{
-	$self->{sign} = $1 || '+';				# no sign => '+'
-	$self->{_n} = $MBI->_new($2 || 0);
+        elsif ($n->isa('Math::BigInt::Lite')) {
+            # TODO: trap NaN, inf
+            $self->{sign} = '+';
+            $self->{sign} = '-' if $$n < 0;
+            $self->{_n} = $MBI->_new(abs($$n)); # "mantissa" = $n
+        }
+        else {
+            Carp::croak(ref($n) . " is not a recognized object format for"
+                        . " Math::BigRat->new");
         }
 
-      if ($d =~ /^([+-]?)0*([0-9]+)\z/)				# second part ok?
-	{
-	$self->{sign} =~ tr/+-/-+/ if ($1 || '') eq '-';	# negate if second part neg.
-	$self->{_d} = $MBI->_new($2 || 0);
+        # now D:
+        if ($d->isa('Math::BigInt')) {
+            # TODO: trap NaN, inf
+            $self->{_d} = $MBI->_copy($d->{value}); # "mantissa" = D
+            # +/+ or -/- => +, +/- or -/+ => -
+            $self->{sign} = $d->{sign} ne $self->{sign} ? '-' : '+';
+        } elsif ($d->isa('Math::BigInt::Lite')) {
+            # TODO: trap NaN, inf
+            $self->{_d} = $MBI->_new(abs($$d)); # "mantissa" = D
+            my $ds = '+';
+            $ds = '-' if $$d < 0;
+            # +/+ or -/- => +, +/- or -/+ => -
+            $self->{sign} = $ds ne $self->{sign} ? '-' : '+';
+        } else {
+            Carp::croak(ref($d) . " is not a recognized object format for"
+                        . " Math::BigRat->new");
         }
 
-      if (!defined $self->{_n} || !defined $self->{_d})
-	{
-        $d = Math::BigInt->new($d,undef,undef) unless ref $d;
-        $n = Math::BigInt->new($n,undef,undef) unless ref $n;
-
-        if ($n->{sign} =~ /^[+-]$/ && $d->{sign} =~ /^[+-]$/)
-	  {
-	  # both parts are ok as integers (weird things like ' 1e0'
-          $self->{_n} = $MBI->_copy($n->{value});
-          $self->{_d} = $MBI->_copy($d->{value});
-          $self->{sign} = $n->{sign};
-          $self->{sign} =~ tr/+-/-+/ if $d->{sign} eq '-';	# -1/-2 => 1/2
-          return $self->bnorm();
-	  }
-
-        $self->{sign} = '+';					# a default sign
-        return $self->bnan() if $n->is_nan() || $d->is_nan();
-
-	# handle inf cases:
-        if ($n->is_inf() || $d->is_inf())
-	  {
-	  if ($n->is_inf())
-	    {
-	    return $self->bnan() if $d->is_inf();		# both are inf => NaN
-	    my $s = '+'; 		# '+inf/+123' or '-inf/-123'
-	    $s = '-' if substr($n->{sign},0,1) ne $d->{sign};
-	    # +-inf/123 => +-inf
-	    return $self->binf($s);
-	    }
-          # 123/inf => 0
-          return $self->bzero();
-	  }
-	}
-      }
-
-    return $self->bnorm();
+        return $self->bnorm();  # normalize (120/100 => 6/5)
     }
 
-  # simple string input
-  if (($n =~ /[\.eE]/) && $n !~ /^0x/)
-    {
-    # looks like a float, quacks like a float, so probably is a float
-    $self->{sign} = 'NaN';
-    local $Math::BigFloat::accuracy = undef;
-    local $Math::BigFloat::precision = undef;
-    $self->_new_from_float(Math::BigFloat->new($n,undef,undef));
+    return $n->copy() if ref $n; # already a BigRat
+
+    if (!defined $n) {
+        $self->{_n} = $MBI->_zero(); # undef => 0
+        $self->{_d} = $MBI->_one();
+        $self->{sign} = '+';
+        return $self;
     }
-  else
-    {
-    # for simple forms, use $MBI directly
-    if ($n =~ /^([+-]?)0*([0-9]+)\z/)
-      {
-      $self->{sign} = $1 || '+';
-      $self->{_n} = $MBI->_new($2 || 0);
-      $self->{_d} = $MBI->_one();
-      }
-    else
-      {
-      my $n = Math::BigInt->new($n,undef,undef);
-      $self->{_n} = $MBI->_copy($n->{value});
-      $self->{_d} = $MBI->_one();
-      $self->{sign} = $n->{sign};
-      return $self->bnan() if $self->{sign} eq 'NaN';
-      return $self->binf($self->{sign}) if $self->{sign} =~ /^[+-]inf$/;
-      }
+
+    # string input with / delimiter
+    if ($n =~ m|\s*/\s*|) {
+        return $class->bnan() if $n =~ m|/.*/|; # 1/2/3 isn't valid
+        return $class->bnan() if $n =~ m|/\s*$|; # 1/ isn't valid
+        ($n, $d) = split (/\//, $n);
+        # try as BigFloats first
+        if (($n =~ /[\.eE]/) || ($d =~ /[\.eE]/)) {
+            local $Math::BigFloat::accuracy = undef;
+            local $Math::BigFloat::precision = undef;
+
+            # one of them looks like a float
+            my $nf = Math::BigFloat->new($n, undef, undef);
+            $self->{sign} = '+';
+            return $self->bnan() if $nf->is_nan();
+
+            $self->{_n} = $MBI->_copy($nf->{_m}); # get mantissa
+
+            # now correct $self->{_n} due to $n
+            my $f = Math::BigFloat->new($d, undef, undef);
+            return $self->bnan() if $f->is_nan();
+            $self->{_d} = $MBI->_copy($f->{_m});
+
+            # calculate the difference between nE and dE
+            my $diff_e = $nf->exponent()->bsub($f->exponent);
+            if ($diff_e->is_negative()) {
+                # < 0: mul d with it
+                $MBI->_lsft($self->{_d}, $MBI->_new($diff_e->babs()), 10);
+            } elsif (!$diff_e->is_zero()) {
+                # > 0: mul n with it
+                $MBI->_lsft($self->{_n}, $MBI->_new($diff_e), 10);
+            }
+        } else {
+            # both d and n look like (big)ints
+
+            $self->{sign} = '+'; # no sign => '+'
+            $self->{_n} = undef;
+            $self->{_d} = undef;
+            if ($n =~ /^([+-]?)0*([0-9]+)\z/) { # first part ok?
+                $self->{sign} = $1 || '+'; # no sign => '+'
+                $self->{_n} = $MBI->_new($2 || 0);
+            }
+
+            if ($d =~ /^([+-]?)0*([0-9]+)\z/) { # second part ok?
+                $self->{sign} =~ tr/+-/-+/ if ($1 || '') eq '-'; # negate if second part neg.
+                $self->{_d} = $MBI->_new($2 || 0);
+            }
+
+            if (!defined $self->{_n} || !defined $self->{_d}) {
+                $d = Math::BigInt->new($d, undef, undef) unless ref $d;
+                $n = Math::BigInt->new($n, undef, undef) unless ref $n;
+
+                if ($n->{sign} =~ /^[+-]$/ && $d->{sign} =~ /^[+-]$/) {
+                    # both parts are ok as integers (weird things like ' 1e0'
+                    $self->{_n} = $MBI->_copy($n->{value});
+                    $self->{_d} = $MBI->_copy($d->{value});
+                    $self->{sign} = $n->{sign};
+                    $self->{sign} =~ tr/+-/-+/ if $d->{sign} eq '-'; # -1/-2 => 1/2
+                    return $self->bnorm();
+                }
+
+                $self->{sign} = '+'; # a default sign
+                return $self->bnan() if $n->is_nan() || $d->is_nan();
+
+                # handle inf cases:
+                if ($n->is_inf() || $d->is_inf()) {
+                    if ($n->is_inf()) {
+                        return $self->bnan() if $d->is_inf(); # both are inf => NaN
+                        my $s = '+'; # '+inf/+123' or '-inf/-123'
+                        $s = '-' if substr($n->{sign}, 0, 1) ne $d->{sign};
+                        # +-inf/123 => +-inf
+                        return $self->binf($s);
+                    }
+                    # 123/inf => 0
+                    return $self->bzero();
+                }
+            }
+        }
+
+        return $self->bnorm();
     }
-  $self->bnorm();
-  }
 
-sub copy
-  {
-  # if two arguments, the first one is the class to "swallow" subclasses
-  my ($c,$x) = @_;
+    # simple string input
+    if (($n =~ /[\.eE]/) && $n !~ /^0x/) {
+        # looks like a float, quacks like a float, so probably is a float
+        $self->{sign} = 'NaN';
+        local $Math::BigFloat::accuracy = undef;
+        local $Math::BigFloat::precision = undef;
+        $self->_new_from_float(Math::BigFloat->new($n, undef, undef));
+    } else {
+        # for simple forms, use $MBI directly
+        if ($n =~ /^([+-]?)0*([0-9]+)\z/) {
+            $self->{sign} = $1 || '+';
+            $self->{_n} = $MBI->_new($2 || 0);
+            $self->{_d} = $MBI->_one();
+        }
+        elsif ($n =~ /^\s*([+-]?)inf(inity)?\s*\z/i) {
+            my $sgn = $1 || '+';
+            $self->{sign} = $sgn . 'inf';   # set a default sign for bstr()
+            $self->binf($sgn);
+        }
 
-  if (scalar @_ == 1)
-    {
-    $x = $_[0];
-    $c = ref($x);
+        else {
+            my $n = Math::BigInt->new($n, undef, undef);
+            $self->{_n} = $MBI->_copy($n->{value});
+            $self->{_d} = $MBI->_one();
+            $self->{sign} = $n->{sign};
+            return $self->bnan() if $self->{sign} eq 'NaN';
+        }
     }
-  return unless ref($x); # only for objects
 
-  my $self = bless {}, $c;
+    $self->bnorm();
+}
 
-  $self->{sign} = $x->{sign};
-  $self->{_d} = $MBI->_copy($x->{_d});
-  $self->{_n} = $MBI->_copy($x->{_n});
-  $self->{_a} = $x->{_a} if defined $x->{_a};
-  $self->{_p} = $x->{_p} if defined $x->{_p};
-  $self;
-  }
+sub copy {
+    my $self    = shift;
+    my $selfref = ref $self;
+    my $class   = $selfref || $self;
+
+    # If called as a class method, the object to copy is the next argument.
+
+    $self = shift() unless $selfref;
+
+    my $copy = bless {}, $class;
+
+    $copy->{sign} = $self->{sign};
+    $copy->{_d} = $MBI->_copy($self->{_d});
+    $copy->{_n} = $MBI->_copy($self->{_n});
+    $copy->{_a} = $self->{_a} if defined $self->{_a};
+    $copy->{_p} = $self->{_p} if defined $self->{_p};
+
+    $copy;
+}
 
 ##############################################################################
 
@@ -401,15 +411,13 @@ sub bnorm
   my ($self,$x) = ref($_[0]) ? (undef,$_[0]) : objectify(1,@_);
 
   # Both parts must be objects of whatever we are using today.
-  if ( my $c = $MBI->_check($x->{_n}) )
+  if (my $c = $MBI->_check($x->{_n}))
     {
-    require Carp; Carp::croak(
-        "n did not pass the self-check ($c) in bnorm()");
+        Carp::croak("n did not pass the self-check ($c) in bnorm()");
     }
-  if ( my $c = $MBI->_check($x->{_d}) )
+  if (my $c = $MBI->_check($x->{_d}))
     {
-    require Carp; Carp::croak(
-        "d did not pass the self-check ($c) in bnorm()");
+        Carp::croak("d did not pass the self-check ($c) in bnorm()");
     }
 
   # no normalize for NaN, inf etc.
@@ -464,7 +472,6 @@ sub _bnan
 
   if ($_trap_nan)
     {
-    require Carp;
     my $class = ref($self);
     # "$self" below will stringify the object, this blows up if $self is a
     # partial object (happens under trap_nan), so fix it beforehand
@@ -483,7 +490,6 @@ sub _binf
 
   if ($_trap_inf)
     {
-    require Carp;
     my $class = ref($self);
     # "$self" below will stringify the object, this blows up if $self is a
     # partial object (happens under trap_nan), so fix it beforehand
@@ -545,16 +551,16 @@ sub badd
   # and bnorm() will then take care of the rest
 
   # 5 * 3
-  $x->{_n} = $MBI->_mul( $x->{_n}, $y->{_d});
+  $x->{_n} = $MBI->_mul($x->{_n}, $y->{_d});
 
   # 7 * 4
-  my $m = $MBI->_mul( $MBI->_copy( $y->{_n} ), $x->{_d} );
+  my $m = $MBI->_mul($MBI->_copy($y->{_n}), $x->{_d});
 
   # 5 * 3 + 7 * 4
-  ($x->{_n}, $x->{sign}) = _e_add( $x->{_n}, $m, $x->{sign}, $y->{sign});
+  ($x->{_n}, $x->{sign}) = _e_add($x->{_n}, $m, $x->{sign}, $y->{sign});
 
   # 4 * 3
-  $x->{_d} = $MBI->_mul( $x->{_d}, $y->{_d});
+  $x->{_d} = $MBI->_mul($x->{_d}, $y->{_d});
 
   # normalize result, and possible round
   $x->bnorm()->round(@r);
@@ -618,8 +624,8 @@ sub bmul
   #  - * - =  -----  = -  = -
   #  4   3    4 * 3    12   6
 
-  $x->{_n} = $MBI->_mul( $x->{_n}, $y->{_n});
-  $x->{_d} = $MBI->_mul( $x->{_d}, $y->{_d});
+  $x->{_n} = $MBI->_mul($x->{_n}, $y->{_n});
+  $x->{_d} = $MBI->_mul($x->{_d}, $y->{_d});
 
   # compute new sign
   $x->{sign} = $x->{sign} eq $y->{sign} ? '+' : '-';
@@ -725,8 +731,8 @@ sub bdiv
   # -  /  - == - * -
   # 4     3    4   1
 
-  $x->{_n} = $MBI->_mul( $x->{_n}, $y->{_d});
-  $x->{_d} = $MBI->_mul( $x->{_d}, $y->{_n});
+  $x->{_n} = $MBI->_mul($x->{_n}, $y->{_d});
+  $x->{_d} = $MBI->_mul($x->{_d}, $y->{_n});
 
   # compute new sign
   $x->{sign} = $x->{sign} eq $y->{sign} ? '+' : '-';
@@ -813,14 +819,14 @@ sub bdec
 
   if ($x->{sign} eq '-')
     {
-    $x->{_n} = $MBI->_add( $x->{_n}, $x->{_d});		# -5/2 => -7/2
+    $x->{_n} = $MBI->_add($x->{_n}, $x->{_d});		# -5/2 => -7/2
     }
   else
     {
     if ($MBI->_acmp($x->{_n},$x->{_d}) < 0)		# n < d?
       {
       # 1/3 -- => -2/3
-      $x->{_n} = $MBI->_sub( $MBI->_copy($x->{_d}), $x->{_n});
+      $x->{_n} = $MBI->_sub($MBI->_copy($x->{_d}), $x->{_n});
       $x->{sign} = '-';
       }
     else
@@ -843,7 +849,7 @@ sub binc
     if ($MBI->_acmp($x->{_n},$x->{_d}) < 0)
       {
       # -1/3 ++ => 2/3 (overflow at 0)
-      $x->{_n} = $MBI->_sub( $MBI->_copy($x->{_d}), $x->{_n});
+      $x->{_n} = $MBI->_sub($MBI->_copy($x->{_d}), $x->{_n});
       $x->{sign} = '+';
       }
     else
@@ -948,9 +954,9 @@ sub parts
   return ($c->binf(),$c->binf()) if $x->{sign} eq '+inf';
   return ($c->binf('-'),$c->binf()) if $x->{sign} eq '-inf';
 
-  my $n = $c->new( $MBI->_str($x->{_n}));
+  my $n = $c->new($MBI->_str($x->{_n}));
   $n->{sign} = $x->{sign};
-  my $d = $c->new( $MBI->_str($x->{_d}));
+  my $d = $c->new($MBI->_str($x->{_d}));
   ($n,$d);
   }
 
@@ -1097,36 +1103,71 @@ sub bpow
   # a/b       =  -\/  (a/b) ** d
 
   # (a/b) ** n == (a ** n) / (b ** n)
-  $MBI->_pow($x->{_n}, $y->{_n} );
-  $MBI->_pow($x->{_d}, $y->{_n} );
+  $MBI->_pow($x->{_n}, $y->{_n});
+  $MBI->_pow($x->{_d}, $y->{_n});
 
   return $x->broot($MBI->_str($y->{_d}),@r);		# n/d => root(n)
   }
 
 sub blog
   {
+  # Return the logarithm of the operand. If a second operand is defined, that
+  # value is used as the base, otherwise the base is assumed to be Euler's
+  # constant.
+
+  # Don't objectify the base, since an undefined base, as in $x->blog() or
+  # $x->blog(undef) signals that the base is Euler's number.
+
   # set up parameters
-  my ($self,$x,$y,@r) = (ref($_[0]),@_);
+  my ($self,$x,$base,@r) = (ref($_[0]),@_);
 
   # objectify is costly, so avoid it
   if ((!ref($_[0])) || (ref($_[0]) ne ref($_[1])))
     {
-    ($self,$x,$y,@r) = objectify(2,$class,@_);
+    ($self,$x,$base,@r) = objectify(1,$class,@_);
     }
 
-  # blog(1,Y) => 0
-  return $x->bzero() if $x->is_one() && $y->{sign} eq '+';
+  return $x if $x->modify('blog');
 
-  # $x <= 0 => NaN
-  return $x->bnan() if $x->is_zero() || $x->{sign} ne '+' || $y->{sign} ne '+';
+  # Handle all exception cases and all trivial cases. I have used Wolfram Alpha
+  # (http://www.wolframalpha.com) as the reference for these cases.
 
-  if ($x->is_int() && $y->is_int())
-    {
-    return $self->new($x->as_number()->blog($y->as_number(),@r));
-    }
+  return $x -> bnan() if $x -> is_nan();
 
-  # do it with floats
-  $x->_new_from_float( $x->_as_float()->blog(Math::BigFloat->new("$y"),@r) );
+  if (defined $base) {
+      $base = $self -> new($base) unless ref $base;
+      if ($base -> is_nan() || $base -> is_one()) {
+          return $x -> bnan();
+      } elsif ($base -> is_inf() || $base -> is_zero()) {
+          return $x -> bnan() if $x -> is_inf() || $x -> is_zero();
+          return $x -> bzero();
+      } elsif ($base -> is_negative()) {            # -inf < base < 0
+          return $x -> bzero() if $x -> is_one();   #     x = 1
+          return $x -> bone()  if $x == $base;      #     x = base
+          return $x -> bnan();                      #     otherwise
+      }
+      return $x -> bone() if $x == $base;           # 0 < base && 0 < x < inf
+  }
+
+  # We now know that the base is either undefined or positive and finite.
+
+  if ($x -> is_inf()) {                 # x = +/-inf
+      my $sign = defined $base && $base < 1 ? '-' : '+';
+      return $x -> binf($sign);
+  } elsif ($x -> is_neg()) {            # -inf < x < 0
+      return $x -> bnan();
+  } elsif ($x -> is_one()) {            # x = 1
+      return $x -> bzero();
+  } elsif ($x -> is_zero()) {           # x = 0
+      my $sign = defined $base && $base < 1 ? '+' : '-';
+      return $x -> binf($sign);
+  }
+
+  # At this point we are done handling all exception cases and trivial cases.
+
+  # Do it with Math::BigFloats and convert back to Math::BigRat.
+  $base = $base -> _as_float() if defined $base;
+  $x -> _new_from_float($x -> _as_float() -> blog($base, @r));
   }
 
 sub bexp
@@ -1252,7 +1293,7 @@ sub bnok
     }
 
   # do it with floats
-  $x->_new_from_float( $x->_as_float()->bnok(Math::BigFloat->new("$y"),@r) );
+  $x->_new_from_float($x->_as_float()->bnok(Math::BigFloat->new("$y"),@r));
   }
 
 sub _float_from_part
@@ -1279,7 +1320,7 @@ sub _as_float
   if ($a != 0 || !$MBI->_is_one($x->{_d}))
     {
     # n/d
-    return scalar Math::BigFloat->new($x->{sign} . $MBI->_str($x->{_n}))->bdiv( $MBI->_str($x->{_d}), $x->accuracy());
+    return scalar Math::BigFloat->new($x->{sign} . $MBI->_str($x->{_n}))->bdiv($MBI->_str($x->{_d}), $x->accuracy());
     }
   # just n
   Math::BigFloat->new($x->{sign} . $MBI->_str($x->{_n}));
@@ -1301,7 +1342,7 @@ sub broot
     }
 
   # do it with floats
-  $x->_new_from_float( $x->_as_float()->broot($y->_as_float(),@r) )->bnorm()->bround(@r);
+  $x->_new_from_float($x->_as_float()->broot($y->_as_float(),@r))->bnorm()->bround(@r);
   }
 
 sub bmodpow
@@ -1367,8 +1408,8 @@ sub bsqrt
   local $Math::BigInt::precision = undef;
   local $Math::BigInt::accuracy = undef;
 
-  $x->{_n} = _float_from_part( $x->{_n} )->bsqrt();
-  $x->{_d} = _float_from_part( $x->{_d} )->bsqrt();
+  $x->{_n} = _float_from_part($x->{_n})->bsqrt();
+  $x->{_d} = _float_from_part($x->{_d})->bsqrt();
 
   # XXX TODO: we probably can optimize this:
 
@@ -1376,19 +1417,19 @@ sub bsqrt
   if ($x->{_d}->{_es} ne '+')
     {
     $x->{_n}->blsft($x->{_d}->exponent()->babs(),10);	# 7.1/4.51 => 7.1/45.1
-    $x->{_d} = $MBI->_copy( $x->{_d}->{_m} );		# 7.1/45.1 => 71/45.1
+    $x->{_d} = $MBI->_copy($x->{_d}->{_m});		# 7.1/45.1 => 71/45.1
     }
   # if sqrt(N) was not integer
   if ($x->{_n}->{_es} ne '+')
     {
     $x->{_d}->blsft($x->{_n}->exponent()->babs(),10);	# 71/45.1 => 710/45.1
-    $x->{_n} = $MBI->_copy( $x->{_n}->{_m} );		# 710/45.1 => 710/451
+    $x->{_n} = $MBI->_copy($x->{_n}->{_m});		# 710/45.1 => 710/451
     }
 
   # convert parts to $MBI again
-  $x->{_n} = $MBI->_lsft( $MBI->_copy( $x->{_n}->{_m} ), $x->{_n}->{_e}, 10)
+  $x->{_n} = $MBI->_lsft($MBI->_copy($x->{_n}->{_m}), $x->{_n}->{_e}, 10)
     if ref($x->{_n}) ne $MBI && ref($x->{_n}) ne 'ARRAY';
-  $x->{_d} = $MBI->_lsft( $MBI->_copy( $x->{_d}->{_m} ), $x->{_d}->{_e}, 10)
+  $x->{_d} = $MBI->_lsft($MBI->_copy($x->{_d}->{_m}), $x->{_d}->{_e}, 10)
     if ref($x->{_d}) ne $MBI && ref($x->{_d}) ne 'ARRAY';
 
   $x->bnorm()->round(@r);
@@ -1400,7 +1441,7 @@ sub blsft
 
   $b = 2 unless defined $b;
   $b = $self->new($b) unless ref ($b);
-  $x->bmul( $b->copy()->bpow($y), @r);
+  $x->bmul($b->copy()->bpow($y), @r);
   $x;
   }
 
@@ -1410,7 +1451,7 @@ sub brsft
 
   $b = 2 unless defined $b;
   $b = $self->new($b) unless ref ($b);
-  $x->bdiv( $b->copy()->bpow($y), @r);
+  $x->bdiv($b->copy()->bpow($y), @r);
   $x;
   }
 
@@ -1468,8 +1509,8 @@ sub bcmp
   return -1 if $xz && $y->{sign} eq '+';                # 0 <=> +y
   return 1 if $yz && $x->{sign} eq '+';                 # +x <=> 0
 
-  my $t = $MBI->_mul( $MBI->_copy($x->{_n}), $y->{_d});
-  my $u = $MBI->_mul( $MBI->_copy($y->{_n}), $x->{_d});
+  my $t = $MBI->_mul($MBI->_copy($x->{_n}), $y->{_d});
+  my $u = $MBI->_mul($MBI->_copy($y->{_n}), $x->{_d});
 
   my $cmp = $MBI->_acmp($t,$u);				# signs are equal
   $cmp = -$cmp if $x->{sign} eq '-';			# both are '-' => reverse
@@ -1497,8 +1538,8 @@ sub bacmp
     return -1;
     }
 
-  my $t = $MBI->_mul( $MBI->_copy($x->{_n}), $y->{_d});
-  my $u = $MBI->_mul( $MBI->_copy($y->{_n}), $x->{_d});
+  my $t = $MBI->_mul($MBI->_copy($x->{_n}), $y->{_d});
+  my $u = $MBI->_mul($MBI->_copy($y->{_n}), $x->{_d});
   $MBI->_acmp($t,$u);					# ignore signs
   }
 
@@ -1527,7 +1568,7 @@ sub as_number
   return Math::BigInt->new($x->{sign}) if $x->{sign} !~ /^[+-]$/;
 
   my $u = Math::BigInt->bzero();
-  $u->{value} = $MBI->_div( $MBI->_copy($x->{_n}), $x->{_d});	# 22/7 => 3
+  $u->{value} = $MBI->_div($MBI->_copy($x->{_n}), $x->{_d});	# 22/7 => 3
   $u->bneg if $x->{sign} eq '-'; # no negative zero
   $u;
   }
@@ -1549,7 +1590,7 @@ sub as_float
   # n
   $u->{_m} = $MBI->_copy($x->{_n});
   $u->{_e} = $MBI->_zero();
-  $u->bdiv( $MBI->_str($x->{_d}), @r);
+  $u->bdiv($MBI->_str($x->{_d}), @r);
   # return $u
   $u;
   }
@@ -1609,7 +1650,7 @@ sub from_oct
     {
     push @parts, Math::BigInt->from_oct($c);
     }
-  $class->new ( @parts );
+  $class->new (@parts);
   }
 
 ##############################################################################
@@ -1622,9 +1663,9 @@ sub import
   my $lib = ''; my @a;
   my $try = 'try';
 
-  for ( my $i = 0; $i < $l ; $i++)
+  for (my $i = 0; $i < $l ; $i++)
     {
-    if ( $_[$i] eq ':constant' )
+    if ($_[$i] eq ':constant')
       {
       # this rest causes overlord er load to step in
       overload::constant float => sub { $self->new(shift); };
@@ -1674,12 +1715,12 @@ sub import
   push @import, $try => $lib if $lib ne '';
 
   # MBI already loaded, so feed it our lib arguments
-  Math::BigInt->import( @import );
+  Math::BigInt->import(@import);
 
   $MBI = Math::BigFloat->config()->{lib};
 
   # register us with MBI to get notified of future lib changes
-  Math::BigInt::_register_callback( $self, sub { $MBI = $_[0]; } );
+  Math::BigInt::_register_callback($self, sub { $MBI = $_[0]; });
 
   # any non :constant stuff is handled by our parent, Exporter (loaded
   # by Math::BigFloat, even if @_ is empty, to give it a chance
